import moment from 'moment'
import { doPaymentThroughAPI, consultPayment } from './PaymentController.js'
import PixRechargeRepository from '../Repositories/PixRechargeRepository.js'
import BaseInfoController from './BaseInfoController.js'
import RechargeController from './RechargeController.js'
import RechargeRepository from '../Repositories/RechargeRepository.js'
import InvoiceRepository from '../Repositories/InvoiceRepository.js'
import { commit, rollback } from './../Connection/QueriesExecutor.js'
import EmailController from './EmailController.js'

let checkPaymentInterval = {}
let checkPaymentTimeout = {}

const getQrcode = async ({ totalValue, isInvoiceMode, consDetails, idBoletos }) => {
    let NRTRANSACAO = ''

    if (isInvoiceMode) {
        NRTRANSACAO = consDetails.CDFILIAL + idBoletos[0]
    } else {
        NRTRANSACAO = await newCode('TRANSACAO', 12)
    }
    
    const response = await doPaymentThroughAPI({ totalValue, mode: 'pix', NRTRANSACAO })

    // console.log(response)

    if (response?.data?.result?.qrCode && response?.data?.result?.nit && response?.data?.result?.pixTimeout) {
        return {
            error: false,
            qrCode: response.data.result.qrCode,
            nit: response.data.result.nit,
            NRTRANSACAO,
            pixTimeout: response.data.result.pixTimeout
        }
    } else {
        return {
            error: true,
            message: response?.data?.messageCode 
        }
    }
}

const doPixRecharge = async () => {
}

const newCode = async function (counterName, size) {
    const baseInfo = await BaseInfoController.getBaseInfo()
    const NRORG = baseInfo[0].NRORG

    const paramsNewCode = {
        NRORGNOVOCODIGO: NRORG,
        NOMETABELA: counterName
    }

    let newCode = await PixRechargeRepository.getNewCode(paramsNewCode)
    if (newCode?.length && size) newCode = newCode.substring(newCode.length - size)
    return newCode
}

const checkPixPayment = async function ({ NRTRANSACAO, nit, pixTimeout, totalValue, rechargeValues, consDetails }) {
    // de 10 em 10 seg manda uma requisição para verificar se o pagamento foi realizado
    // fazer isso durante pixTimeout segundos
    const connectionKey = consDetails.CDCLIENTE + consDetails.CDCONSUMIDOR
    const { NMRESPCONS, DSEMAILCONS } = consDetails

    return new Promise((resolve, reject) => {
        let hasResolved = false

        checkPaymentInterval[connectionKey] = setInterval(async () => {
            const response = await checkPaymentStatus(nit, connectionKey)
            // console.log('response', response)
            
            if (response?.paymentSuccess) {
                // pagamento realizado
                clearInterval(checkPaymentInterval[connectionKey])
                delete checkPaymentInterval[connectionKey]

                // clear timeout
                clearTimeout(checkPaymentTimeout[connectionKey])
                delete checkPaymentTimeout[connectionKey]

                hasResolved = true
                
                // atualizar o saldo do cartão e apagar os dados da tabela temporaria
                const updateResponse = await updateBalance ({ NRTRANSACAO, connectionKey })
                console.log('updateResponse', updateResponse)
                if (updateResponse) {
                    await commit({ connectionKey })

                    // send confirmation email
                    const transactionValue = parseFloat(totalValue).toLocaleString('pt-br', { style: 'currency', currency: 'BRL' })
                    const transactionDate = moment().format('DD/MM/YYYY HH:mm:ss')
                    const studentsNames = rechargeValues.map((family) => family.NMCONSUMIDOR).join(', ')

                    const emailData = {
                        name: NMRESPCONS,
                        studentsNames,
                        transactionValue,
                        transactionDate
                    }

                    const emailResult = await EmailController.sendEmail({ DSEMAILCONS, subject: 'Comprovante de Recarga do Cartão', consDetails, htmlFunction: 'getPixPaymentConfirmationEmail', functionParams: emailData})

                    if (emailResult.accepted?.length > 0) {
                        resolve({
                            error: false,
                            message: 'Pagamento realizado com sucesso. Comprovante enviado por e-mail.'
                        })
                    }

                    resolve({
                        error: false,
                        message: 'Pagamento realizado com sucesso. Não foi possível enviar o comprovante por e-mail.'
                    })
                }
                else {
                    await rollback({ connectionKey })
                    resolve({
                        error: true,
                        message: 'Pagamento confirmado, mas ocorreu um erro ao atualizar o saldo do cartão. Reinicie o aplicativo e, caso o erro persista, entre em contato com o suporte.'
                    })
                }
                return 
            } else {
                if (response?.status === 'DENIED') {
                    clearInterval(checkPaymentInterval[connectionKey])
                    delete checkPaymentInterval[connectionKey]

                    // clear timeout
                    clearTimeout(checkPaymentTimeout[connectionKey])
                    delete checkPaymentTimeout[connectionKey]

                    hasResolved = true
                    await RechargeRepository.deleteTempRecharge({ NRTRANSACAO }, true)
                    resolve({
                        error: true,
                        message: 'Pagamento negado. Tente novamente.'
                    })
                    return
                } 
                else if (response?.status === 'EXPIRED') {
                    clearInterval(checkPaymentInterval[connectionKey])
                    delete checkPaymentInterval[connectionKey]

                    // clear timeout
                    clearTimeout(checkPaymentTimeout[connectionKey])
                    delete checkPaymentTimeout[connectionKey]

                    hasResolved = true
                    await RechargeRepository.deleteTempRecharge({ NRTRANSACAO }, true)
                    resolve({
                        error: true,
                        message: 'Tempo limite para pagamento excedido. Tente novamente.'
                    })
                    return
                } 
                else {
                    if (hasResolved) return
                }
            }

        }, 10000) // 10 segundos

        checkPaymentTimeout[connectionKey] = setTimeout(async () => {
            clearInterval(checkPaymentInterval[connectionKey])
            delete checkPaymentInterval[connectionKey]

            // clear timeout
            clearTimeout(checkPaymentTimeout[connectionKey])
            delete checkPaymentTimeout[connectionKey]

            if (!hasResolved) {
                hasResolved = true

                // apagar os dados da tabela temporaria
                await RechargeRepository.deleteTempRecharge({ NRTRANSACAO }, true)

                resolve({
                    error: true,
                    message: 'Tempo limite para pagamento excedido. Tente novamente.'
                })
            }
        }, pixTimeout)
    })
}

const checkPixInvoicePayment = async function ({ idBoletos, nit, pixTimeout, consDetails, totalValue }) {
    // de 10 em 10 seg manda uma requisição para verificar se o pagamento foi realizado
    // fazer isso durante pixTimeout segundos
    const connectionKey = consDetails.CDCLIENTE + consDetails.CDCONSUMIDOR

    return new Promise((resolve, reject) => {
        let hasResolved = false

        checkPaymentInterval[connectionKey] = setInterval(async () => {
            const response = await checkPaymentStatus(nit, connectionKey)
            
            if (response?.paymentSuccess) {
                clearInterval(checkPaymentInterval[connectionKey])
                delete checkPaymentInterval[connectionKey]

                // clear timeout
                clearTimeout(checkPaymentTimeout[connectionKey])
                delete checkPaymentTimeout[connectionKey]

                hasResolved = true
                
                if (idBoletos?.length > 0) {
                    // Atualiza a tabela do financeiro
                    for (let i = 0; i < idBoletos.length; i++) {
                        await InvoiceRepository.updatePixIncome({ NIT: nit, NRLANBOLREC: idBoletos[i], CDFILIAL: consDetails.CDFILIAL })
                    }
                }

                // send confirmation email
                const transactionValue = parseFloat(totalValue).toLocaleString('pt-br', { style: 'currency', currency: 'BRL' })
                const transactionDate = moment().format('DD/MM/YYYY HH:mm:ss')
                let docsNumbers = idBoletos.join(', ')

                const emailData = { name: consDetails.NMRESPCONS, transactionValue, transactionDate, docsNumbers }

                const emailResult = await EmailController.sendEmail({
                    DSEMAILCONS: consDetails.DSEMAILCONS,
                    subject: 'Comprovante de Pagamento de Título',
                    consDetails,
                    htmlFunction: 'getPixInvoicePaymentConfirmationEmail',
                    functionParams: emailData
                })

                if (emailResult.accepted?.length > 0) {
                    resolve({ error: false, message: 'Pagamento realizado com sucesso. Comprovante enviado por e-mail.' })
                }

                resolve({ error: false, message: 'Pagamento realizado com sucesso. Não foi possível enviar o comprovante por e-mail.' })
                return
            } else {
                if (response?.status === 'DENIED') {
                    clearInterval(checkPaymentInterval[connectionKey])
                    delete checkPaymentInterval[connectionKey]

                    // clear timeout
                    clearTimeout(checkPaymentTimeout[connectionKey])
                    delete checkPaymentTimeout[connectionKey]

                    hasResolved = true
                    resolve({ error: true, message: 'Pagamento negado. Tente novamente.' })
                    return
                } else if (response?.status === 'EXPIRED') {
                    clearInterval(checkPaymentInterval[connectionKey])
                    delete checkPaymentInterval[connectionKey]

                    // clear timeout
                    clearTimeout(checkPaymentTimeout[connectionKey])
                    delete checkPaymentTimeout[connectionKey]

                    hasResolved = true
                    resolve({ error: true, message: 'Tempo limite para pagamento excedido. Tente novamente.' })
                    return
                } else {
                    if (hasResolved) return
                }
            }

        }, 10000) // 10 segundos
    
        checkPaymentTimeout[connectionKey] = setTimeout(async () => {
            clearInterval(checkPaymentInterval[connectionKey])
            delete checkPaymentInterval[connectionKey]

            // clear timeout
            clearTimeout(checkPaymentTimeout[connectionKey])
            delete checkPaymentTimeout[connectionKey]

            if (!hasResolved) {
                hasResolved = true
                resolve({ error: true, message: 'Tempo limite para pagamento excedido. Tente novamente.' })
            }
        }, pixTimeout)
    })
}

const checkPaymentStatus = async function (nit, connectionKey) {
    console.log(nit)
    const response = await consultPayment(nit)
    console.log(response?.data?.result?.status)
    if (response?.data?.result?.status === 'CONFIRMED') {
        clearInterval(checkPaymentInterval[connectionKey])
        delete checkPaymentInterval[connectionKey]
        return {
            paymentSuccess: true,
            status: response?.data?.result?.status
        }
    }
    return {
        paymentSuccess: false,
        status: response?.data?.result?.status
    }
}

const updateBalance = async function ({ NRTRANSACAO, connectionKey }) {
    // atualizar o saldo do cartão e apagar os dados da tabela temporaria
    try {
        const tempRecharge = await PixRechargeRepository.getTempRecharge(NRTRANSACAO, false, connectionKey)
        console.log(tempRecharge)
        const { CDCLIENTE, CDFILIAL, CDTIPOCONS, TIPORECE, NSU, RECHARGE_VALUES } = tempRecharge
        const rechargeValues = JSON.parse(RECHARGE_VALUES)
        console.log( 'rechargeValues', rechargeValues)

        await RechargeController.updateBalance(rechargeValues, { CDCLIENTE, CDFILIAL, CDTIPOCONS}, TIPORECE, NRTRANSACAO, NSU, connectionKey)

        await RechargeRepository.deleteTempRecharge({ NRTRANSACAO }, false, connectionKey)

        return true
    } catch (error) {
        console.log(error)
        return false
    }
    
}

const saveTempRecharge = async function ({rechargeValues, NRTRANSACAO, nit, consDetails, tipoRece = null, nsu = null, }) {
    const { CDCLIENTE, CDFILIAL, CDTIPOCONS, CDCONSUMIDOR } = consDetails
    const result = await PixRechargeRepository.saveTempRecharge({ rechargeValues, CDCLIENTE, CDFILIAL, CDTIPOCONS, NRTRANSACAO, tipoRece, nsu, nit, CDCONSUMIDOR })
    return result
}

const checkOpenedPayment = async function ({ consDetails }) {
    const connectionKey = consDetails.CDCLIENTE + consDetails.CDCONSUMIDOR
    console.log(connectionKey)
    if (checkPaymentInterval[connectionKey]) return true
}

/**
    Criar função que salva rechargeValues, CDCLIENTE, CDFILIAL, CDTIPOCONS, NRTRANSACAO, tipoRece, nsu e nit em uma tabela temporaria

    - Caso o pagamento seja realizado
    Quando o pagamento for realizado, verificar essa tabela e fazer a recarga de cada um dos consumidores
    Após a recarga, apagar os dados da tabela temporaria

    - Caso o pagamento não seja realizado
    Apagar os dados da tabela temporaria
    
    - Caso o pagamento seja realizado, mas a recarga não
    Dar rollback na transação
    Ao fazer login, verificar se existe alguma recarga pendente e fazer a recarga, olhando pelo NIT da transação se o pagamento foi realizado
    Após a recarga, apagar os dados da tabela temporaria
 */

    /** RECHARGE_VALUES will store json
     * create temporary table script oracle
    CREATE TABLE TEMP_RECHARGE (
        CDCLIENTE VARCHAR(21),
        CDFILIAL VARCHAR(4),
        CDTIPOCONS VARCHAR2 (2)
        NRTRANSACAO VARCHAR2(12) PRIMARY KEY,
        TIPORECE NUMBER(10),
        NSU VARCHAR2(30),
        NIT VARCHAR2(64),
        CDCONSUMIDOR VARCHAR2(21),
        CDFAMILISALD VARCHAR2(3),
        RECHARGE_VALUES VARCHAR2(4000)
    ) ON COMMIT PRESERVE ROWS; 

    on commit preserve rows - não apaga os dados da tabela ao fazer commit
     */

export default { getQrcode, doPixRecharge, checkPixPayment, checkPixInvoicePayment, saveTempRecharge, checkOpenedPayment }